package jp.co.argo21.nautica.workflow.soap;

import javax.annotation.Resource;
import javax.jws.WebService;
import javax.xml.ws.WebServiceContext;

import org.apache.log4j.Logger;

import jp.co.argo21.nautica.workflow.client.WorkflowServiceAccessor;
import jp.co.argo21.nautica.workflow.soap.message.Attribute;
import jp.co.argo21.nautica.workflow.soap.message.Filter;
import jp.co.argo21.nautica.workflow.soap.message.Participant;
import jp.co.argo21.nautica.workflow.soap.message.WorkItem;
import jp.co.argo21.nautica.workflow.soap.message.WorkItemState;
import jp.co.argo21.nautica.workflow.util.StringManager;
import jp.co.argo21.nautica.workflow.wfmc.AttributeAssignmentFailedException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidAttributeException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidFilterException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidSessionException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidSourceUserException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidStateException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidTargetUserException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidWorkItemException;
import jp.co.argo21.nautica.workflow.wfmc.TransitionNotAllowedException;

/**
 * Web サービスとして公開するインタフェースの SOAP バインディング実装クラスである。
 * ワークフローに対するクライアンクラスである。
 */  
@WebService(serviceName = "WorkItemHandler",
        portName="WorkItemHandlerPort",
        endpointInterface = "jp.co.argo21.nautica.workflow.soap.WorkItemHandler",
        targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap",
        wsdlLocation = "WEB-INF/wsdl/WorkItemHandler.wsdl")
public class WorkItemHandlerSoapBindingImpl implements WorkItemHandler {

	/** ログ出力用オブジェクト */
	private static Logger log = Logger.getLogger(
			DefinitionRepositorySoapBindingImpl.class);
	
	@Resource
    private WebServiceContext context;
	
	/**
	 * 作業項目属性の割り当てを行う。
	 *
	 * @param session セッションID
	 * @param wid 作業項目ID
	 * @param attr 属性
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidWorkItemException 作業項目IDが有効でない場合
	 * @throws InvalidAttributeException 属性が有効でない場合
	 * @throws AttributeAssignmentFailedException 属性の割り当てに失敗した場合
	 */
	@Override
	public void assignWorkItemAttribute(String session, String wid,
			Attribute attr) throws InvalidSessionException, InvalidWorkItemException, InvalidAttributeException, AttributeAssignmentFailedException {

		String I4001 = StringManager.get("I4001")
			+ "(SessionID=" + session
			+ ", WorkItemID=" + wid 
			+ ", 属性=" + attr + ")";
		log.info(I4001);
		
		try {
			WorkflowServiceAccessor accessor = SingletonServices.getInstance().getAccessor(session);
			jp.co.argo21.nautica.workflow.wfmc.WorkItemHandler wih = accessor.getWorkItemHandler();
			wih.assignWorkItemAttribute(session, wid, attr.toWfmcAttribute());
		} catch (InvalidSessionException e) {
			String errMsg = StringManager.get("E1015");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidWorkItemException e) {
			String errMsg = StringManager.get("E0115");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidAttributeException e) {
			String errMsg = StringManager.get("E0233");
			log.error(errMsg, e);
			throw e;
		} catch (AttributeAssignmentFailedException e) {
			String errMsg = StringManager.get("E0232");
			log.error(errMsg, e);
			throw e;
		}
		
		String I4002 = StringManager.get("I4002");
	   log.info(I4002);
		
	}

	/**
	 * 作業項目状態の変更を行う。
	 *
	 * @param session セッションID
	 * @param wid 作業項目ID
	 * @param state 作業項目状態
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidWorkItemException 作業項目IDが有効でない場合
	 * @throws InvalidStateException 指定された状態が有効でない場合
	 * @throws TransitionNotAllowedException 状態の変更が許可されていない場合
	 */
	@Override
	public void changeWorkItemState(String session, String wid,
			WorkItemState state) throws InvalidWorkItemException, InvalidStateException, TransitionNotAllowedException, InvalidSessionException {
		
		String I4001 = StringManager.get("I4001")
			+ "(SessionID=" + session
			+ ", WorkItemID=" + wid 
			+ ", WorkItemState=" + state + ")";
		log.info(I4001);
		
		try {
			WorkflowServiceAccessor accessor = SingletonServices.getInstance().getAccessor(session);
			jp.co.argo21.nautica.workflow.wfmc.WorkItemHandler wih = accessor.getWorkItemHandler();
			wih.changeWorkItemState(session, wid, state.toWfmcWorkItemState());
		} catch (InvalidSessionException e) {
			String errMsg = StringManager.get("E1015");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidWorkItemException e) {
			String errMsg = StringManager.get("E0115");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidStateException e) {
			String errMsg = StringManager.get("E0234");
			log.error(errMsg, e);
			throw e;
		} catch (TransitionNotAllowedException e) {
			String errMsg = StringManager.get("E0050");
			log.error(errMsg, e);
			throw e;
		}
		
		String I4002 = StringManager.get("I4002");
	   log.info(I4002);
		
	}

	/**
	 * 作業項目の完了を行う。
	 *
	 * @param session セッションID
	 * @param wid 作業項目ID
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidWorkItemException 作業項目IDが有効でない場合
	 */
	@Override
	public void completeWorkItem(String session, String wid) throws InvalidSessionException, InvalidWorkItemException {
		
		String I4001 = StringManager.get("I4001")
			+ "(SessionID=" + session
			+ ", WorkItemID=" + wid + ")";
		log.info(I4001);
		
		try {
			WorkflowServiceAccessor accessor = SingletonServices.getInstance().getAccessor(session);
			jp.co.argo21.nautica.workflow.wfmc.WorkItemHandler wih = accessor.getWorkItemHandler();
			wih.completeWorkItem(session, wid);
		} catch (InvalidSessionException e) {
			String errMsg = StringManager.get("E1015");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidWorkItemException e) {
			String errMsg = StringManager.get("E0115");
			log.error(errMsg, e);
			throw e;
		}
		
		String I4002 = StringManager.get("I4002");
	   log.info(I4002);
		
	}

	/**
	 * 作業項目の取得を行う。
	 *
	 * @param session セッションID
	 * @param wid 作業項目ID
	 * @return 作業項目
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidWorkItemException 作業項目IDが有効でない場合
	 */
	@Override
	public WorkItem getWorkItem(String session, String wid) throws InvalidSessionException, InvalidWorkItemException {
		
		String I4001 = StringManager.get("I4001")
			+ "(SessionID=" + session
			+ ", WorkItemID=" + wid + ")";
		log.info(I4001);
		
		WorkItem wi = null;
		try {
			WorkflowServiceAccessor accessor = SingletonServices.getInstance().getAccessor(session);
			jp.co.argo21.nautica.workflow.wfmc.WorkItemHandler wih = accessor.getWorkItemHandler();
			wi = new WorkItem(wih.getWorkItem(session, wid));
		} catch (InvalidSessionException e) {
			String errMsg = StringManager.get("E1015");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidWorkItemException e) {
			String errMsg = StringManager.get("E0115");
			log.error(errMsg, e);
			throw e;
		}
		
		String I4002 = StringManager.get("I4002")
			+ "(WorkItem=" + wi + ")";
   	log.info(I4002);
		return wi;
		
	}

	/**
	 * 作業項目属性の取得を行う。
	 *
	 * @param session セッションID
	 * @param wid 作業項目ID
	 * @param name 属性名
	 * @return 属性
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidWorkItemException 作業項目IDが有効でない場合
	 * @throws InvalidAttributeException 属性が有効でない場合
	 */
	@Override
	public Attribute getWorkItemAttributeValue(String session, String wid,
			String name) throws InvalidSessionException, InvalidWorkItemException, InvalidAttributeException {

		String I4001 = StringManager.get("I4001")
			+ "(SessionID=" + session
			+ ", WorkItemID=" + wid 
			+ ", 属性名=" + name + ")";
		log.info(I4001);
		
		Attribute attr = null;
		try {
			WorkflowServiceAccessor accessor = SingletonServices.getInstance().getAccessor(session);
			jp.co.argo21.nautica.workflow.wfmc.WorkItemHandler wih = accessor.getWorkItemHandler();
			attr = new Attribute(
					wih.getWorkItemAttributeValue(session, wid, name));
		} catch (InvalidSessionException e) {
			String errMsg = StringManager.get("E1015");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidWorkItemException e) {
			String errMsg = StringManager.get("E0115");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidAttributeException e) {
			String errMsg = StringManager.get("E0233");
			log.error(errMsg, e);
			throw e;
		}
		
		String I4002 = StringManager.get("I4002")
			+ "(属性=" + attr + ")";
   	log.info(I4002);
		return attr;
		
	}

	/**
	 * 作業項目属性一覧の取得を行う。
	 * 利用可能なフィルターはAttributeFilterのみとする。
	 *
	 * @param session セッションID
	 * @param wid 作業項目ID
	 * @param filter 検索用フィルター
	 * @return 作業項目属性一覧
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidWorkItemException 作業項目IDが有効でない場合
	 * @throws InvalidFilterException 指定されたフィルターが有効でない場合
	 */
	@Override
	public Attribute[] getWorkItemAttributes(String session, String wid,
			Filter[] filter) throws InvalidSessionException, InvalidWorkItemException, InvalidFilterException {

		String I4001 = StringManager.get("I4001")
			+ "(SessionID=" + session
			+ ", WorkItemID=" + wid 
			+ ", Filter=" + filter + ")";
		log.info(I4001);
		
		Attribute[] attrs = null;
		try {
			WorkflowServiceAccessor accessor = SingletonServices.getInstance().getAccessor(session);
			jp.co.argo21.nautica.workflow.wfmc.WorkItemHandler wih = accessor.getWorkItemHandler();
			attrs = SoapMessageConverter.convertToSoapMessage(
					wih.getWorkItemAttributes(
							session, 
							wid, 
							SoapMessageConverter.convertToGroupFilter(filter)));
		} catch (InvalidSessionException e) {
			String errMsg = StringManager.get("E1015");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidWorkItemException e) {
			String errMsg = StringManager.get("E0115");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidFilterException e) {
			String errMsg = StringManager.get("E0004");
			log.error(errMsg, e);
			throw e;
		}
		
		String I4002 = StringManager.get("I4002") 
			+ "(属性の配列=" + attrs + ")";
   	log.info(I4002);
		return attrs;
		
	}

	/**
	 * 現在の作業項目が遷移可能な状態の一覧を返す。
	 * 状態がない場合は、空の配列を返す。
	 * 利用可能なフィルターはStateFilterのみとする。
	 *
	 * @param session セッションID
	 * @param wid 作業項目ID
	 * @param filter 検索用フィルター
	 * @return 作業項目状態一覧
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidWorkItemException 作業項目IDが有効でない場合
	 * @throws InvalidFilterException 指定されたフィルターが有効でない場合
	 */
	@Override
	public WorkItemState[] getWorkItemStates(String session, String wid,
			Filter[] filter) throws InvalidSessionException, InvalidFilterException, InvalidWorkItemException {

		String I4001 = StringManager.get("I4001")
			+ "(SessionID=" + session
			+ ", WorkItemID=" + wid 
			+ ", Filter=" + filter + ")";
		log.info(I4001);
		
		WorkItemState[] wiss = null;
		try {
			WorkflowServiceAccessor accessor = SingletonServices.getInstance().getAccessor(session);
			jp.co.argo21.nautica.workflow.wfmc.WorkItemHandler wih = accessor.getWorkItemHandler();
			wiss = SoapMessageConverter.convertToSoapMessage(
					wih.getWorkItemStates(
							session, 
							wid, 
							SoapMessageConverter.convertToStateFilter(filter)));
		} catch (InvalidSessionException e) {
			String errMsg = StringManager.get("E1015");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidWorkItemException e) {
			String errMsg = StringManager.get("E0115");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidFilterException e) {
			String errMsg = StringManager.get("E0004");
			log.error(errMsg, e);
			throw e;
		}
		
		String I4002 = StringManager.get("I4002") 
			+ "(WorkItemStates=" + wiss + ")";
   	log.info(I4002);
		return wiss;
		
	}

	/**
	 * 作業項目一覧の取得を行う。
	 *
	 * @param session セッションID
	 * @param filter 検索用フィルター
	 * @return 作業項目一覧
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidFilterException 指定されたフィルターが有効でない場合
	 */
	@Override
	public WorkItem[] getWorkItems(String session, Filter[] filter) throws InvalidSessionException, InvalidFilterException {
		
		String I4001 = StringManager.get("I4001") 
			+ "(SessionID=" + session 
			+ ", Filter=" + filter + ")";
		log.info(I4001);
		
		WorkItem[] wis = null;
		try {
			WorkflowServiceAccessor accessor = SingletonServices.getInstance().getAccessor(session);
			jp.co.argo21.nautica.workflow.wfmc.WorkItemHandler wih = accessor.getWorkItemHandler();
			wis = SoapMessageConverter.convertToSoapMessage(
					wih.getWorkItems(
							session, 
							SoapMessageConverter.convertToGroupFilter(filter)));
		} catch (InvalidSessionException e) {
			String errMsg = StringManager.get("E1015");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidFilterException e) {
			String errMsg = StringManager.get("E0004");
			log.error(errMsg, e);
			throw e;
		}
		
		String I4002 = StringManager.get("I4002") 
			+ "(WorkItems=" + wis + ")";
   	log.info(I4002);
		return wis;
		
	}

	/**
	 * 作業項目の再割り当てを行う。
	 *
	 * @param session セッションID
	 * @param sourceUser 割り当て前のユーザ
	 * @param targetUser 割り当て後のユーザ
	 * @param wid 作業項目ID
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidWorkItemException 作業項目IDが有効でない場合
	 * @throws InvalidTargetUserException 割り当て先ユーザが有効でない場合
	 * @throws InvalidSourceUserException 割り当て前ユーザが有効でない場合
	 */
	@Override
	public void reassignWorkItem(String session, Participant sourceUser,
			Participant targetUser, String wid) throws InvalidSessionException, InvalidWorkItemException, InvalidSourceUserException, InvalidTargetUserException {

		String I4001 = StringManager.get("I4001")
			+ "(SessionID=" + session
			+ ", SourceUser=" + sourceUser 
			+ ", TargetUser=" + targetUser  
			+ ", WorkItemID=" + wid + ")";
		log.info(I4001);
		
		try {
			WorkflowServiceAccessor accessor = SingletonServices.getInstance().getAccessor(session);
			jp.co.argo21.nautica.workflow.wfmc.WorkItemHandler wih = accessor.getWorkItemHandler();
			wih.reassignWorkItem(
			        session, 
			        sourceUser.toWfmcParticipant(), 
			        targetUser.toWfmcParticipant(), 
			        wid);
		} catch (InvalidSessionException e) {
			String errMsg = StringManager.get("E1015");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidWorkItemException e) {
			String errMsg = StringManager.get("E0115");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidSourceUserException e) {
			String errMsg = StringManager.get("E0229");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidTargetUserException e) {
			String errMsg = StringManager.get("E0229");
			log.error(errMsg, e);
			throw e;
		}
		
		String I4002 = StringManager.get("I4002");
	   log.info(I4002);
		
	}

	public WebServiceContext getContext() {
		return context;
	}
	
}